
-module(libnet_test).
-define(DRV, libnet_drv).

%% "erl +A5 -pz . -s libnet_test regression -s erlang halt", the

-export([regression/0]).

regression() ->
    {ok, Port} = ?DRV:start(),
    {ok, 1} = ?DRV:debug(Port, 1),
    r1(Port),
    io:format("All regression tests PASSED.\n"),
    ok.

r1(P) ->
    ok = ?DRV:null(P),

    {ok, S1} = libnet_drv:libnet_open_raw_sock(P, 0),
    
    {ok, 0} = libnet_drv:test_send_ping(P, "10.1.1.1", "10.1.1.253", S1),

    {ok, 0} = libnet_drv:libnet_close_raw_sock(P, S1),
    {error, badarg} = libnet_drv:libnet_close_raw_sock(P, S1),

    %% QQQ My desktop box's Ethernet interface is "rl0"
    {ok, R1} = libnet_drv:libnet_open_link_interface(P, "rl0"),
    {ok, 1} = libnet_drv:libnet_close_link_interface(P, R1),
    {error, badarg} = libnet_drv:libnet_close_link_interface(P, R1),

    {ok, {_, "rl0"}} = libnet_drv:libnet_select_device(P, "rl0"),
    {ok, {_, "rl0"}} = libnet_drv:my_libnet_select_device_any(P),
    {error, _} = libnet_drv:libnet_select_device(P, "xl0"),
    
    {ok, {Ping1}} =
	libnet_drv:test_make_icmp_echo_no_checksum(P,
						   "10.1.1.1", "10.1.1.254"),
    Ping1Len = size(Ping1),
    {ok, {Ping2}} = libnet_drv:libnet_do_checksum(P, Ping1, 1, 8),
    Ping2Len = size(Ping2),
    {error, badarg} = libnet_drv:libnet_write_ip(P, S1, Ping1),
    {error, badarg} = libnet_drv:libnet_write_ip(P, S1, Ping2),
    {ok, S2} = libnet_drv:libnet_open_raw_sock(P, 0),
    {ok, Ping1Len} = libnet_drv:libnet_write_ip(P, S1, Ping1),
    {ok, Ping2Len} = libnet_drv:libnet_write_ip(P, S1, Ping2),

    %% Sanity check of contiguousifying lists of binaries!
    <<Ping2a:7/binary, Ping2b:9/binary, Ping2c/binary>> = Ping2,
    {ok, {Ping2}} = 
	libnet_drv:libnet_do_checksum(P, [Ping2a, Ping2b, Ping2c], 1, 8),


    ok.
