
#include	<stdio.h>
#include	<sys/time.h>

#include	<pcap.h>
#include	<net/bpf.h>
#include	<sys/ioctl.h>

static int	count = 0;
static pcap_t	*p;

void do_one_packet(u_char *user, const struct pcap_pkthdr *h, const u_char *p);
void fmt_packet(int count, const u_char *pkt);

int
main(int argc, char *argv[])
{
    char		errbuf[PCAP_ERRBUF_SIZE];
    struct timeval	tv;
    u_int		immediate = 1;
    int			last_count = count;

    if (argc != 2) {
	fprintf(stderr, "usage: %s interface-name\n", argv[0]);
	exit(1);
    }
    errbuf[0] = '\0';
    if ((p = pcap_open_live(argv[1], 32768, 1, 2000, errbuf)) == NULL) {
	fprintf(stderr, "open_offline: %s\n", errbuf);
	exit(1);
    }
    if (strlen(errbuf) != 0) {
	fprintf(stderr, "WARNING: %s\n", errbuf);
    }
    if (ioctl(pcap_fileno(p), BIOCIMMEDIATE, &immediate) < 0) {
	perror("ioctl(BIOCIMMEDIATE)");
	exit(1);
    }

    while (1) {
	pcap_dispatch(p, -1, do_one_packet, NULL);
	fprintf(stderr, "%d ", count - last_count);
	last_count = count;
    }
    pcap_close(p);


    exit(0);
}

void
do_one_packet(u_char *user, const struct pcap_pkthdr *h, const u_char *pkt)
{
    u_int	caplen = h->caplen;
    struct pcap_stat ps;

    count++;
    if (caplen < 12) {
	fprintf(stderr, "Pkt too short!\n");
	exit(2);
    }
    if (count % 1000 == 0) {
	if (pcap_stats(p, &ps) == 0) {
	    fprintf(stderr, "\n received %u dropped %u\n", ps.ps_recv, ps.ps_drop);
	} else {
	    perror("XXX MOO!");
	}
    }
#ifdef	XXXQQQ
    fmt_packet(count, pkt);
#endif	/* XXXQQQ */
}

void
fmt_packet(int count, const u_char *pkt)
{
    struct timeval	tv;

    gettimeofday(&tv, NULL);
    printf("packet %d: %ld.%06ld dst ether = %02x:%02x:%02x:%02x:%02x:%02x\n",
	   count, tv.tv_sec, tv.tv_usec, pkt[0], pkt[1], pkt[2], pkt[3], pkt[4], pkt[5]);
    printf("packet %d: %ld.%06ld src ether = %02x:%02x:%02x:%02x:%02x:%02x\n",
	   count, tv.tv_sec, tv.tv_usec, pkt[6], pkt[7], pkt[8], pkt[9], pkt[10], pkt[11]);
    printf("\n");
}
