-module('libpcap_test').
-define(DRV, libpcap_drv).

-export([regression/0]).

%% Change these interface names to suit your local environment
-define(IFACE_EXIST, "rl0").
-define(IFACE_NOTEXIST, "xl0").
%% Path to the "test-captures" directory
-define(TEST_CAPTURES_DIR,
	"/user/fritchie/src/e-d/drtk/examples/libpcap/test-captures").

regression() ->
    {ok, Port} = ?DRV:start(),
    {ok, 1} = ?DRV:debug(Port, 1),
    r1(Port),
    io:format("All regression tests PASSED.\n"),
    ok.

r1(P) ->
    ok = ?DRV:null(P),

    {ok, PC1} = ?DRV:pcap_open_offline(P, ?TEST_CAPTURES_DIR ++ "/tcpdump.1"),
    {ok, {{Pkt1_TS_s, Pkt1_TS_us}, Pkt1}} = ?DRV:pcap_next(P, PC1),
    {ok, {{Pkt2_TS_s, Pkt2_TS_us}, Pkt2}} = ?DRV:pcap_next(P, PC1),
    {ok, {{Pkt3_TS_s, Pkt3_TS_us}, Pkt3}} = ?DRV:pcap_next(P, PC1),
    {ok, {{Pkt4_TS_s, Pkt4_TS_us}, Pkt4}} = ?DRV:pcap_next(P, PC1),
    {ok, {{Pkt5_TS_s, Pkt5_TS_us}, Pkt5}} = ?DRV:pcap_next(P, PC1),
    {error, {}} = ?DRV:pcap_next(P, PC1),
    {ok, _} = libpcap_drv:pcap_is_swapped(P, PC1),
    {ok, _} = ?DRV:pcap_major_version(P, PC1),
    {ok, _} = ?DRV:pcap_minor_version(P, PC1),
    {ok, 0} = ?DRV:pcap_close(P, PC1),
    {error, badarg} = ?DRV:pcap_close(P, PC1),

    {ok, {Iface1}} = ?DRV:pcap_lookupdev(P),
    
    %% QQQ On my test platform, ?IFACE_NOTEXIST does not exist, "rl0" does.
    {error, _} = ?DRV:pcap_lookupnet(P, ?IFACE_NOTEXIST),
    {ok, {ExistAddr, ExistMask}} = ?DRV:pcap_lookupnet(P, ?IFACE_EXIST),

    {ok, {_BigList}} = ?DRV:pcap_findalldevs(P),

    %% Intentional leaking here
    {ok, PC98} = ?DRV:pcap_open_offline(P, ?TEST_CAPTURES_DIR ++ "/tcpdump.1"),
    {ok, PC99} = ?DRV:pcap_open_offline(P, ?TEST_CAPTURES_DIR ++ "/tcpdump.1"),

    {error, _} = ?DRV:pcap_open_live(P, ?IFACE_NOTEXIST, 1514, 1, 2000),
    {ok, PC2} = ?DRV:pcap_open_live(P, ?IFACE_EXIST, 1514, 1, 120 * 1000),
    {ok, 0} = ?DRV:my_set_immediate(P, PC2, 0),
    {ok, 0} = ?DRV:my_set_immediate(P, PC2, 1),
    ReadPkts = 5,
    io:format("\nPlease generate at least ~w packets of network traffic\n",
	      [ReadPkts]),
    ok = read_packets(P, PC2, ReadPkts),
    io:format("\nGot ~w packets.  Thank you!\n\n", [ReadPkts]),
    {ok, 0} = ?DRV:pcap_getnonblock(P, PC2),
    {ok, 0} = ?DRV:pcap_setnonblock(P, PC2, 1),
    {ok, 1} = ?DRV:pcap_getnonblock(P, PC2),
    
    %% QQQ My "rl0" is an Ethernet device = 1
    {ok, 1} = libpcap_drv:pcap_datalink(P, PC2),
    {ok, 1514} = libpcap_drv:pcap_snapshot(P, PC2),
    {ok, {PS_Recv, PS_Drop}} = libpcap_drv:pcap_stats(P, PC2),
    if
	PS_Recv < ReadPkts -> throw({failed, only_read, PS_Recv});	    
	PS_Drop > 0 -> throw({failed, dropped, PS_Drop});
	true -> ok
    end,

    %% The pcap_stats test must be done before setting this filter
    {error, _} = ?DRV:my_pcap_compile(P, PC2, "BOGUS", 1, ExistMask),
    {ok, _} = ?DRV:pcap_geterr(P, PC2),
    {ok, BPF1} = ?DRV:my_pcap_compile(P, PC2, "arp or icmp", 1, ExistMask),
    {ok, 0} = libpcap_drv:pcap_setfilter(P, PC2, BPF1),
    {ok, 0} = libpcap_drv:my_pcap_freecode(P, BPF1),
    {error, badarg} = libpcap_drv:my_pcap_freecode(P, BPF1),

    {ok, 0} = ?DRV:pcap_close(P, PC1),
    
    ok.

read_packets(P, PC, 0) when port(P) ->
    ok;
read_packets(P, PC, N) when port(P) ->
    {ok, {_Pkt_TS, Pkt}} = ?DRV:pcap_next(P, PC),
    io:format("Got packet ~w, ~w bytes\n", [N, size(Pkt)]),
    read_packets(P, PC, N - 1).

