%%%----------------------------------------------------------------------
%%% File    : libnet_drv.erl
%%% Summary : Copyright (c) 2002, Scott Lystig Fritchie.  All rights reserved.
%%%
%%%
%%% NOTICE: This file was generated by the tools of the Erlang Driver
%%%         toolkit.  Do not edit this file by hand unless you know
%%%         what you're doing!
%%%
%%% Copyright (c) 2002, Scott Lystig Fritchie.  All rights reserved.
%%% See the file "LICENSE" at the top of the source distribution for
%%% full license terms.
%%%
%%%----------------------------------------------------------------------

-module(libnet_drv).
-include("libnet_drv.hrl").

%% External exports
-export([start/0, shutdown/1, debug/2]).
-export([
         null/1, 
         libnet_open_raw_sock/2, 
         libnet_close_raw_sock/2, 
         my_libnet_select_device_any/1, 
         libnet_select_device/2, 
         libnet_open_link_interface/2, 
         libnet_close_link_interface/2, 
         libnet_write_ip/3, 
         libnet_write_link_layer/4, 
         libnet_do_checksum/4, 
         libnet_ip_check/2, 
         test_send_ping/4, 
         test_make_icmp_echo_no_checksum/3
        ]).

start() ->
    {ok, Path} = load_path(),
    erl_ddll:start(),
    ok = erl_ddll:load_driver(Path, ?DRV_NAME),
    case open_port({spawn, ?DRV_NAME}, []) of
        P when port(P) ->
            {ok, P};
        Err ->
            Err
    end.

shutdown(Port) when port(Port) ->
    erlang:port_close(Port).

debug(Port, Flags) when port(Port), integer(Flags) ->
    erlang:port_command(Port, <<?_DEBUG, Flags:32>>),
    get_port_reply(Port).

null(Port
        ) when port(Port) -> % TODO: Add additional constraints here
    B = <<?_NULL>>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

libnet_open_raw_sock(Port,
     Protocol
        ) when port(Port) -> % TODO: Add additional constraints here
    B = <<?_LIBNET_OPEN_RAW_SOCK,
          Protocol:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

libnet_close_raw_sock(Port,
     Fd
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_raw_sock, Raw_SockIndex} = Fd,
    B = <<?_LIBNET_CLOSE_RAW_SOCK,
            Raw_SockIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

my_libnet_select_device_any(Port
        ) when port(Port) -> % TODO: Add additional constraints here
    B = <<?_MY_LIBNET_SELECT_DEVICE_ANY>>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

libnet_select_device(Port,
     Device
        ) when port(Port) -> % TODO: Add additional constraints here
    {DeviceBinOrList, DeviceLen} = serialize_contiguously(Device, 1),
    B = [ <<?_LIBNET_SELECT_DEVICE,
            DeviceLen:32/integer>>,             % I/O list length
          DeviceBinOrList,
          <<
        >> ],
    erlang:port_command(Port, B),
    get_port_reply(Port).

libnet_open_link_interface(Port,
     Device
        ) when port(Port) -> % TODO: Add additional constraints here
    {DeviceBinOrList, DeviceLen} = serialize_contiguously(Device, 1),
    B = [ <<?_LIBNET_OPEN_LINK_INTERFACE,
            DeviceLen:32/integer>>,             % I/O list length
          DeviceBinOrList,
          <<
        >> ],
    erlang:port_command(Port, B),
    get_port_reply(Port).

libnet_close_link_interface(Port,
     L
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_link_int, Link_IntIndex} = L,
    B = <<?_LIBNET_CLOSE_LINK_INTERFACE,
            Link_IntIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

libnet_write_ip(Port,
     Fd, 
     Buf
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_raw_sock, Raw_SockIndex} = Fd,
    {BufBinOrList, BufLen} = serialize_contiguously(Buf, 0),
    B = [ <<?_LIBNET_WRITE_IP,
            Raw_SockIndex:32/integer, 
            BufLen:32/integer>>,                % I/O list length
          BufBinOrList,
          <<
        >> ],
    erlang:port_command(Port, B),
    get_port_reply(Port).

libnet_write_link_layer(Port,
     L, 
     Device, 
     Buf
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_link_int, Link_IntIndex} = L,
    {DeviceBinOrList, DeviceLen} = serialize_contiguously(Device, 1),
    {BufBinOrList, BufLen} = serialize_contiguously(Buf, 0),
    B = [ <<?_LIBNET_WRITE_LINK_LAYER,
            Link_IntIndex:32/integer, 
            DeviceLen:32/integer>>,             % I/O list length
          DeviceBinOrList,
          <<
            BufLen:32/integer>>,                % I/O list length
          BufBinOrList,
          <<
        >> ],
    erlang:port_command(Port, B),
    get_port_reply(Port).

libnet_do_checksum(Port,
     Buf, 
     Protocol, 
     Len
        ) when port(Port) -> % TODO: Add additional constraints here
    {BufBinOrList, BufLen} = serialize_contiguously(Buf, 0),
    B = [ <<?_LIBNET_DO_CHECKSUM,
            BufLen:32/integer>>,                % I/O list length
          BufBinOrList,
          <<
          Protocol:32/integer, 
          Len:32/integer
        >> ],
    erlang:port_command(Port, B),
    get_port_reply(Port).

libnet_ip_check(Port,
     Buf_U_Short
        ) when port(Port) -> % TODO: Add additional constraints here
    {Buf_U_ShortBinOrList, Buf_U_ShortLen} = serialize_contiguously(Buf_U_Short, 0),
    B = [ <<?_LIBNET_IP_CHECK,
            Buf_U_ShortLen:32/integer>>,        % I/O list length
          Buf_U_ShortBinOrList,
          <<
        >> ],
    erlang:port_command(Port, B),
    get_port_reply(Port).

test_send_ping(Port,
     Src, 
     Dst, 
     Fd
        ) when port(Port) -> % TODO: Add additional constraints here
    {SrcBinOrList, SrcLen} = serialize_contiguously(Src, 1),
    {DstBinOrList, DstLen} = serialize_contiguously(Dst, 1),
    {valmap_raw_sock, Raw_SockIndex} = Fd,
    B = [ <<?_TEST_SEND_PING,
            SrcLen:32/integer>>,                % I/O list length
          SrcBinOrList,
          <<
            DstLen:32/integer>>,                % I/O list length
          DstBinOrList,
          <<
            Raw_SockIndex:32/integer
        >> ],
    erlang:port_command(Port, B),
    get_port_reply(Port).

test_make_icmp_echo_no_checksum(Port,
     Src, 
     Dst
        ) when port(Port) -> % TODO: Add additional constraints here
    {SrcBinOrList, SrcLen} = serialize_contiguously(Src, 1),
    {DstBinOrList, DstLen} = serialize_contiguously(Dst, 1),
    B = [ <<?_TEST_MAKE_ICMP_ECHO_NO_CHECKSUM,
            SrcLen:32/integer>>,                % I/O list length
          SrcBinOrList,
          <<
            DstLen:32/integer>>,                % I/O list length
          DstBinOrList,
          <<
        >> ],
    erlang:port_command(Port, B),
    get_port_reply(Port).

    
%%%
%%% Internal functions.
%%%

load_path() ->
    File = ?DRV_NAME ++ ".so",
    case lists:filter(fun(D) ->
                              case file:read_file_info(D ++ "/" ++ File) of
                                  {ok, _} -> true;
                                  _ -> false
                              end
                      end, code:get_path()) of
        [Dir|_] ->
            {ok, Dir};
        [] ->
            io:format("Error: ~s not found in code path
", [File]),
            {error, enoent}
    end.

get_port_reply(Port) ->
    receive
        {Port, ok} -> ok;
        {Port, M} -> throw(M);          % QQQ!
        {Port, ok, M} -> {ok, M};
        %% {Port, ok, M1, M2} -> {ok, M1, M2};
        {Port, error, Reason} -> {error, Reason}
        ;
        M -> io:format("QQQ XXX got ~w
", [M]), M
    end.

%%% QQQ io_list_len() is an extremely useful function.  BEAM has got this
%%% implemented quite efficiently in C.  It would be *fabulous* to be able
%%% to use it from Erlang via a BIF.

io_list_len(B) when binary(B) -> {B, size(B)};
io_list_len(L) when list(L) -> io_list_len(L, 0).
io_list_len([H|T], N) ->
    if
        H >= 0, H =< 255 -> io_list_len(T, N+1);
        list(H) -> io_list_len(T, io_list_len(H,N));
        binary(H) -> io_list_len(T, size(H) + N);
        true -> throw({error, partial_len, N})
    end;
io_list_len(H, N) when binary(H) -> 
    size(H) + N;
io_list_len([], N) -> 
    N.

%%% QQQ We need to make the binary thing we're passing in contiguous
%%% because the C function we're calling is expecting a single
%%% contiguous buffer.  If IOList is ["Hello, ", <<"World">>, "!"],
%%% that binary in the middle element will end up with the argument
%%% spanning three parts of an ErlIOVec.  If that happens, then we'd
%%% have to have the driver do the dirty work of putting the argument
%%% into a single contiguous buffer.  Frankly, we're lazy, and this
%%% code is short and won't be much slower than doing it in C.

serialize_contiguously(B, 0) when binary(B) ->
    {B, size(B)};
serialize_contiguously([B], 0) when binary(B) ->
    {B, size(B)};
serialize_contiguously(IOList, 1) ->
    serialize_contiguously([IOList, 0], 0);
serialize_contiguously(IOList, 0) ->
    B = list_to_binary(IOList),
    {B, size(B)}.
