%%%----------------------------------------------------------------------
%%% File    : libpcap_drv.erl
%%% Summary : Copyright (c) 2002, Scott Lystig Fritchie.  All rights reserved.
%%%
%%%
%%% NOTICE: This file was generated by the tools of the Erlang Driver
%%%         toolkit.  Do not edit this file by hand unless you know
%%%         what you're doing!
%%%
%%% Copyright (c) 2002, Scott Lystig Fritchie.  All rights reserved.
%%% See the file "LICENSE" at the top of the source distribution for
%%% full license terms.
%%%
%%%----------------------------------------------------------------------

-module(libpcap_drv).
-include("libpcap_drv.hrl").

%% External exports
-export([start/0, shutdown/1, debug/2]).
-export([
         null/1, 
         pcap_open_live/5, 
         pcap_open_offline/2, 
         my_set_immediate/3, 
         pcap_setnonblock/3, 
         pcap_getnonblock/2, 
         pcap_findalldevs/1, 
         pcap_lookupdev/1, 
         pcap_lookupnet/2, 
         my_pcap_compile/5, 
         pcap_setfilter/3, 
         my_pcap_freecode/2, 
         pcap_next/2, 
         pcap_datalink/2, 
         pcap_snapshot/2, 
         pcap_is_swapped/2, 
         pcap_major_version/2, 
         pcap_minor_version/2, 
         pcap_stats/2, 
         pcap_fileno/2, 
         pcap_geterr/2, 
         pcap_close/2
        ]).

start() ->
    {ok, Path} = load_path(),
    erl_ddll:start(),
    ok = erl_ddll:load_driver(Path, ?DRV_NAME),
    case open_port({spawn, ?DRV_NAME}, []) of
        P when port(P) ->
            {ok, P};
        Err ->
            Err
    end.

shutdown(Port) when port(Port) ->
    erlang:port_close(Port).

debug(Port, Flags) when port(Port), integer(Flags) ->
    erlang:port_command(Port, <<?_DEBUG, Flags:32>>),
    get_port_reply(Port).

null(Port
        ) when port(Port) -> % TODO: Add additional constraints here
    B = <<?_NULL>>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_open_live(Port,
     Device, 
     Snaplen, 
     Promisc, 
     To_Ms
        ) when port(Port) -> % TODO: Add additional constraints here
    {DeviceBinOrList, DeviceLen} = serialize_contiguously(Device, 1),
    B = [ <<?_PCAP_OPEN_LIVE,
            DeviceLen:32/integer>>,             % I/O list length
          DeviceBinOrList,
          <<
          Snaplen:32/integer, 
          Promisc:32/integer, 
          To_Ms:32/integer
        >> ],
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_open_offline(Port,
     Fname
        ) when port(Port) -> % TODO: Add additional constraints here
    {FnameBinOrList, FnameLen} = serialize_contiguously(Fname, 1),
    B = [ <<?_PCAP_OPEN_OFFLINE,
            FnameLen:32/integer>>,              % I/O list length
          FnameBinOrList,
          <<
        >> ],
    erlang:port_command(Port, B),
    get_port_reply(Port).

my_set_immediate(Port,
     P, 
     Immediate
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_MY_SET_IMMEDIATE,
            Pcap_TIndex:32/integer, 
          Immediate:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_setnonblock(Port,
     P, 
     Nonblock
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_PCAP_SETNONBLOCK,
            Pcap_TIndex:32/integer, 
          Nonblock:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_getnonblock(Port,
     P
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_PCAP_GETNONBLOCK,
            Pcap_TIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_findalldevs(Port
        ) when port(Port) -> % TODO: Add additional constraints here
    B = <<?_PCAP_FINDALLDEVS>>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_lookupdev(Port
        ) when port(Port) -> % TODO: Add additional constraints here
    B = <<?_PCAP_LOOKUPDEV>>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_lookupnet(Port,
     Device
        ) when port(Port) -> % TODO: Add additional constraints here
    {DeviceBinOrList, DeviceLen} = serialize_contiguously(Device, 1),
    B = [ <<?_PCAP_LOOKUPNET,
            DeviceLen:32/integer>>,             % I/O list length
          DeviceBinOrList,
          <<
        >> ],
    erlang:port_command(Port, B),
    get_port_reply(Port).

my_pcap_compile(Port,
     P, 
     Str, 
     Optimize, 
     Netmask
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    {StrBinOrList, StrLen} = serialize_contiguously(Str, 1),
    B = [ <<?_MY_PCAP_COMPILE,
            Pcap_TIndex:32/integer, 
            StrLen:32/integer>>,                % I/O list length
          StrBinOrList,
          <<
          Optimize:32/integer, 
          Netmask:32/unsigned-integer
        >> ],
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_setfilter(Port,
     P, 
     Fp
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    {valmap_bpf_program, Bpf_ProgramIndex} = Fp,
    B = <<?_PCAP_SETFILTER,
            Pcap_TIndex:32/integer, 
            Bpf_ProgramIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

my_pcap_freecode(Port,
     Fp
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_bpf_program, Bpf_ProgramIndex} = Fp,
    B = <<?_MY_PCAP_FREECODE,
            Bpf_ProgramIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_next(Port,
     P
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_PCAP_NEXT,
            Pcap_TIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_datalink(Port,
     P
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_PCAP_DATALINK,
            Pcap_TIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_snapshot(Port,
     P
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_PCAP_SNAPSHOT,
            Pcap_TIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_is_swapped(Port,
     P
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_PCAP_IS_SWAPPED,
            Pcap_TIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_major_version(Port,
     P
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_PCAP_MAJOR_VERSION,
            Pcap_TIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_minor_version(Port,
     P
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_PCAP_MINOR_VERSION,
            Pcap_TIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_stats(Port,
     P
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_PCAP_STATS,
            Pcap_TIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_fileno(Port,
     P
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_PCAP_FILENO,
            Pcap_TIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_geterr(Port,
     P
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_PCAP_GETERR,
            Pcap_TIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

pcap_close(Port,
     P
        ) when port(Port) -> % TODO: Add additional constraints here
    {valmap_pcap_t, Pcap_TIndex} = P,
    B = <<?_PCAP_CLOSE,
            Pcap_TIndex:32/integer
        >>,
    erlang:port_command(Port, B),
    get_port_reply(Port).

    
%%%
%%% Internal functions.
%%%

load_path() ->
    File = ?DRV_NAME ++ ".so",
    case lists:filter(fun(D) ->
                              case file:read_file_info(D ++ "/" ++ File) of
                                  {ok, _} -> true;
                                  _ -> false
                              end
                      end, code:get_path()) of
        [Dir|_] ->
            {ok, Dir};
        [] ->
            io:format("Error: ~s not found in code path
", [File]),
            {error, enoent}
    end.

get_port_reply(Port) ->
    receive
        {Port, ok} -> ok;
        {Port, M} -> throw(M);          % QQQ!
        {Port, ok, M} -> {ok, M};
        %% {Port, ok, M1, M2} -> {ok, M1, M2};
        {Port, error, Reason} -> {error, Reason}
        ;
        M -> io:format("QQQ XXX got ~w
", [M]), M
    end.

%%% QQQ io_list_len() is an extremely useful function.  BEAM has got this
%%% implemented quite efficiently in C.  It would be *fabulous* to be able
%%% to use it from Erlang via a BIF.

io_list_len(B) when binary(B) -> {B, size(B)};
io_list_len(L) when list(L) -> io_list_len(L, 0).
io_list_len([H|T], N) ->
    if
        H >= 0, H =< 255 -> io_list_len(T, N+1);
        list(H) -> io_list_len(T, io_list_len(H,N));
        binary(H) -> io_list_len(T, size(H) + N);
        true -> throw({error, partial_len, N})
    end;
io_list_len(H, N) when binary(H) -> 
    size(H) + N;
io_list_len([], N) -> 
    N.

%%% QQQ We need to make the binary thing we're passing in contiguous
%%% because the C function we're calling is expecting a single
%%% contiguous buffer.  If IOList is ["Hello, ", <<"World">>, "!"],
%%% that binary in the middle element will end up with the argument
%%% spanning three parts of an ErlIOVec.  If that happens, then we'd
%%% have to have the driver do the dirty work of putting the argument
%%% into a single contiguous buffer.  Frankly, we're lazy, and this
%%% code is short and won't be much slower than doing it in C.

serialize_contiguously(B, 0) when binary(B) ->
    {B, size(B)};
serialize_contiguously([B], 0) when binary(B) ->
    {B, size(B)};
serialize_contiguously(IOList, 1) ->
    serialize_contiguously([IOList, 0], 0);
serialize_contiguously(IOList, 0) ->
    B = list_to_binary(IOList),
    {B, size(B)}.
