
-module(simple1_test).
-define(DRV, simple1_drv).

-export([regression/0]).

regression() ->
    {ok, Port} = ?DRV:start(),
    {ok, 1} = ?DRV:debug(Port, 1),
    r1(Port),
    io:format("All regression tests PASSED.\n"),
    ok.

r1(P) ->
    ok = ?DRV:null(P),

    {ok, 0} = ?DRV:bigendian(P),		% I'm using an x86, YMMV!
    
    {ok, 0} = ?DRV:doubleit(P, 0),
    {ok, -2} = ?DRV:doubleit(P, -1),
    {ok, 42} = ?DRV:doubleit(P, 21),
    {ok, 664} = ?DRV:doubleit(P, 332),
    {error, -100} = ?DRV:doubleit(P, 333),
    {ok, -668} = ?DRV:doubleit(P, -334),

    {ok, 0} = ?DRV:addthem(P, -4, 4),
    {ok, 0} = ?DRV:addthem(P, 4, -4),
    {ok, -8} = ?DRV:addthem(P, -4, -4),
    {ok, 8} = ?DRV:addthem(P, 4, 4),
    {ok, -1} = ?DRV:addthem(P, 0, -1),

    {ok, -16} = ?DRV:multthem(P, -4, 4),
    {ok, -16} = ?DRV:multthem(P, 4, -4),
    {ok, 16} = ?DRV:multthem(P, -4, -4),
    {ok, 16} = ?DRV:multthem(P, 4, 4),
    {ok, 0} = ?DRV:multthem(P, 0, -1),
    {ok, -4} = ?DRV:multthem(P, 4, -1),
    {ok, 4} = ?DRV:multthem(P, 4, 1),
    {ok, 252} = ?DRV:multthem(P, 4, 999999),	% 2nd arg is serialized 8 bits!
    {ok, -252} = ?DRV:multthem(P, 4, -999999),	% Ditto!

    {ok, FD1} = ?DRV:open(P, "/etc/termcap", 0),
    io:format("open: FD1 = ~w\n", [FD1]),
    {ok, FD2} = ?DRV:open(P, "/etc/termcap", 0),
    io:format("open: FD2 = ~w\n", [FD2]),
    %% QQQ ENOENT = 2 on many/most UNIXen ... NT?  VxWorks?
    {error, 2} = ?DRV:open(P, "/etc/DOESNOTEXIST", 0),
    io:format("open: FD3 = -1\n"),
    {ok, 0} = ?DRV:close(P, FD1),
    {error, badarg} = ?DRV:close(P, FD1),

    %% QQQ Note that there's a file descriptor leak via FD1 and FD2 
    %% remaining unclosed.  If you run the regression test via
    %% "erl +A5 -pz . -s simple1_test regression -s erlang halt", the
    %% erlang:halt() function doesn't give ports the opportunity
    %% to shut down gracefully.  If you leave off the second function,
    %% "-s erlang halt", then you'll see that the port is closed
    %% and the port cleanup takes place.
    
    %% QQQ Magic numbers from FreeBSD: O_WRONLY | O_CREAT | O_TRUNC
    {ok, FD10} = ?DRV:open3(P, "./FOO-open-testfile", 16#0001 bor 16#0200 bor 16#0400, 8#0644),
    S_0 = "Hello, world!\nIt's a great day to code in Erlang\n",
    B_0 = list_to_binary(lists:duplicate(400, S_0)),
    B_0_Len = size(B_0),
    io:format("QQQ ZZZ 0\n"),
    {ok, B_0_Len} = ?DRV:write(P, FD10, B_0),
    io:format("QQQ ZZZ 1\n"),
    %% Intentionally don't close FD10
    {ok, FD11} = ?DRV:open(P, "./FOO-open-testfile", 16#000),
    {ok, {B_00}} = ?DRV:read(P, FD11, 85),
    85 = size(B_00),
    io:format("QQQ ZZZ 2, B_00 = ~s\n", [binary_to_list(B_00)]),

    {ok, 42} = ?DRV:foofoo(P, "22", "333", "4444"),
    {ok, 42} = ?DRV:foofoo(P, "Hello", "world!", "Mondo foofoo"),
    
    {ok, T1} = ?DRV:ctime_returnhack(P, 0),
    io:format("ctime_returnhack: T1 = ~w\n", [T1]),
    {ok, T2} = ?DRV:ctime_returnhack(P, 0),
    io:format("ctime_returnhack: T2 = ~w\n", [T2]),
    {ok, T3} = ?DRV:ctime_returnhack(P, 0),
    io:format("ctime_returnhack: T3 = ~w\n", [T3]),

    %% QQQ These tests are sensitive to your local system's timezone setting!
    %% QQQ Mine is CSTCDT aka UTC -0600 (UTC -0500 for daylight savings)
    TB1 = list_to_binary("Wed Dec 31 18:00:00 1969\n"),
    {ok, TB1} = ?DRV:ctime_time_t(P, 0),
    TB2 = list_to_binary("Fri Jul 12 18:58:35 2002\n"),
    {ok, TB2} = ?DRV:ctime_time_t(P, 1026518315),

    File1 = "./FOO-fopen-testfile",
    {ok, FILEP0} = ?DRV:fopen(P, File1, "w+"),
    S_1 = "Hello, world!\nIt's a great day to code in Erlang\n",
    B_1 = list_to_binary(lists:duplicate(400, S_1)),
    <<B_M1:80/binary, B_M2:800/binary, _/binary>> = B_1,
    B_1_Len = size(B_1),
    io:format("QQQ got this far!\n"),
    {ok, B_1_Len} = ?DRV:fwrite(P, B_1, FILEP0),
    {ok, 0} = ?DRV:fclose(P, FILEP0),
    {ok, FILEP1} = ?DRV:fopen(P, File1, "r"),
    {ok, {B_21}} = ?DRV:fread(P, 1, 80, FILEP1),
    80 = size(B_21),
    {ok, {B_22}} = ?DRV:fread(P, 1, 800, FILEP1),
    800 = size(B_22),
    io:format("QQQ made it!\n"),
    %%
    {ok, FILEP2} = ?DRV:fopen(P, "/etc/termcap", "r"), %QQQ file ! exist?
    {ok, {S1}} = ?DRV:fread_string(P, 1, 80, FILEP2),
    io:format("fread_string: S1 -->~s<--\n", [S1]),
    {ok, 0} = ?DRV:fclose(P, FILEP2),
    {ok, -1} = ?DRV:fclose(P, FILEP2),

    {ok, B_stat0} = ?DRV:lstat(P, "/etc/termcap"),
    io:format("QQQ B_stat0 = ~w\n", [B_stat0]),
    {ok, B_stat1} = ?DRV:lstat(P, "/etc/termcap"),
    io:format("QQQ B_stat1 = ~w\n", [B_stat1]),
    {ok, B_stat2} = ?DRV:lstat(P, "/etc/termcap"),
    io:format("QQQ B_stat2 = ~w\n", [B_stat2]),
    {ok, B_stat3} = ?DRV:lstat(P, "/etc/termcap"),
    io:format("QQQ B_stat3 = ~w\n", [B_stat3]),

    ok.
